@@ -20,7 +20,9 @@
 #define GROONGA_IN_H
 
 #ifdef __GNUC__
+#ifndef _GNU_SOURCE
 #define _GNU_SOURCE
+#endif /* _GNU_SOURCE */
 #endif /* __GNUC__ */
 
 #if defined(__NetBSD__) && !defined(_NETBSD_SOURCE)
@@ -40,6 +42,10 @@
 #endif
 #endif /* USE_AIO */
 
+#ifdef __cplusplus
+#define __STDC_LIMIT_MACROS
+#endif
+
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
@@ -76,7 +82,7 @@
 #ifdef GROONGA_MAIN
 #  define GRN_VAR __declspec(dllimport)
 #else
-#  define GRN_VAR __declspec(dllexport)
+#  define GRN_VAR __declspec(dllexport) extern
 #endif /* GROONGA_MAIN */
 #else
 #  define GRN_API
