@@ -295,7 +295,7 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
     if (taintable) { grn_db_touch(ctx, DB_OBJ(table_)->db); }
     grn_obj_unlink(ctx, table_);
   } else {
-    ERR(GRN_INVALID_ARGUMENT, "invalid table name: %.*s", table_len, table);
+    ERR(GRN_INVALID_ARGUMENT, "invalid table name: <%.*s>", table_len, table);
   }
   if (match_escalation_threshold_len) {
     grn_ctx_set_match_escalation_threshold(ctx, original_threshold);
@@ -438,6 +438,9 @@ grn_parse_table_create_flags(grn_ctx *ctx, const char *nptr, const char *end)
     } else if (!memcmp(nptr, "TABLE_PAT_KEY", 13)) {
       flags |= GRN_OBJ_TABLE_PAT_KEY;
       nptr += 13;
+    } else if (!memcmp(nptr, "TABLE_DAT_KEY", 13)) {
+      flags |= GRN_OBJ_TABLE_DAT_KEY;
+      nptr += 13;
     } else if (!memcmp(nptr, "TABLE_NO_KEY", 12)) {
       flags |= GRN_OBJ_TABLE_NO_KEY;
       nptr += 12;
@@ -507,6 +510,9 @@ grn_table_create_flags_to_text(grn_ctx *ctx, grn_obj *buf, grn_obj_flags flags)
   case GRN_OBJ_TABLE_PAT_KEY:
     GRN_TEXT_PUTS(ctx, buf, "TABLE_PAT_KEY");
     break;
+  case GRN_OBJ_TABLE_DAT_KEY:
+    GRN_TEXT_PUTS(ctx, buf, "TABLE_DAT_KEY");
+    break;
   case GRN_OBJ_TABLE_NO_KEY:
     GRN_TEXT_PUTS(ctx, buf, "TABLE_NO_KEY");
     break;
@@ -638,7 +644,7 @@ proc_table_remove(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_d
 static grn_obj *
 proc_column_create(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
-  grn_obj *column, *table, *type;
+  grn_obj *column, *table = NULL, *type = NULL;
   const char *rest;
   grn_obj_flags flags = grn_atoi(GRN_TEXT_VALUE(VAR(2)),
                                  GRN_BULK_CURR(VAR(2)), &rest);
@@ -647,11 +653,27 @@ proc_column_create(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_
                                           GRN_BULK_CURR(VAR(2)));
     if (ctx->rc) { goto exit; }
   }
-  table = grn_ctx_get(ctx, GRN_TEXT_VALUE(VAR(0)),
-                      GRN_TEXT_LEN(VAR(0)));
+  table = grn_ctx_get(ctx, GRN_TEXT_VALUE(VAR(0)), GRN_TEXT_LEN(VAR(0)));
+  if (!table) {
+    ERR(GRN_INVALID_ARGUMENT,
+        "[column][create]: table doesn't exist: <%.*s>",
+        GRN_TEXT_LEN(VAR(0)), GRN_TEXT_VALUE(VAR(0))) ;
+    goto exit;
+  }
   type = grn_ctx_get(ctx, GRN_TEXT_VALUE(VAR(3)),
                      GRN_TEXT_LEN(VAR(3)));
-  if (GRN_TEXT_LEN(VAR(1))) { flags |= GRN_OBJ_PERSISTENT; }
+  if (!type) {
+    ERR(GRN_INVALID_ARGUMENT,
+        "[column][create]: type doesn't exist: <%.*s>",
+        GRN_TEXT_LEN(VAR(3)), GRN_TEXT_VALUE(VAR(3))) ;
+    goto exit;
+  }
+  if (GRN_TEXT_LEN(VAR(1))) {
+    flags |= GRN_OBJ_PERSISTENT;
+  } else {
+    ERR(GRN_INVALID_ARGUMENT, "[column][create]: name is missing");
+    goto exit;
+  }
   column = grn_column_create(ctx, table,
                              GRN_TEXT_VALUE(VAR(1)),
                              GRN_TEXT_LEN(VAR(1)),
@@ -688,6 +710,8 @@ proc_column_create(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_
   }
 exit:
   GRN_OUTPUT_BOOL(!ctx->rc);
+  if (table) { grn_obj_unlink(ctx, table); }
+  if (type) { grn_obj_unlink(ctx, type); }
   return NULL;
 }
 
@@ -906,6 +930,7 @@ print_tableinfo(grn_ctx *ctx, grn_obj *table)
   switch (table->header.type) {
   case GRN_TABLE_HASH_KEY:
   case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
   case GRN_TABLE_NO_KEY:
   case GRN_TABLE_VIEW:
     break;
@@ -1246,8 +1271,8 @@ proc_get_resolve_parameters(grn_ctx *ctx, grn_user_data *user_data, grn_obj **ta
 
   key_text = GRN_TEXT_VALUE(VAR(1));
   key_length = GRN_TEXT_LEN(VAR(1));
-  id_text = GRN_TEXT_VALUE(VAR(4));
-  id_length = GRN_TEXT_LEN(VAR(4));
+  id_text = GRN_TEXT_VALUE(VAR(3));
+  id_length = GRN_TEXT_LEN(VAR(3));
   switch ((*table)->header.type) {
   case GRN_TABLE_NO_KEY:
     if (key_length) {
@@ -1274,6 +1299,7 @@ proc_get_resolve_parameters(grn_ctx *ctx, grn_user_data *user_data, grn_obj **ta
     break;
   case GRN_TABLE_HASH_KEY:
   case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
   case GRN_TABLE_VIEW:
     if (key_length && id_length) {
       ERR(GRN_INVALID_ARGUMENT,
@@ -1432,6 +1458,7 @@ dump_index_column_sources(grn_ctx *ctx, grn_obj *outbuf, grn_obj *column)
       if (i) { GRN_TEXT_PUTC(ctx, outbuf, ','); }
       switch (source->header.type) {
       case GRN_TABLE_PAT_KEY:
+      case GRN_TABLE_DAT_KEY:
       case GRN_TABLE_HASH_KEY:
       case GRN_TABLE_VIEW:
         GRN_TEXT_PUTS(ctx, outbuf, "_key");
@@ -1523,6 +1550,7 @@ reference_column_p(grn_ctx *ctx, grn_obj *column)
   switch (range->header.type) {
   case GRN_TABLE_HASH_KEY:
   case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
   case GRN_TABLE_NO_KEY:
   case GRN_TABLE_VIEW:
     return GRN_TRUE;
@@ -1596,6 +1624,7 @@ dump_records(grn_ctx *ctx, grn_obj *outbuf, grn_obj *table)
   switch (table->header.type) {
   case GRN_TABLE_HASH_KEY:
   case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
   case GRN_TABLE_NO_KEY:
     break;
   case GRN_TABLE_VIEW:
@@ -1631,7 +1660,8 @@ dump_records(grn_ctx *ctx, grn_obj *outbuf, grn_obj *table)
     GRN_BULK_REWIND(&column_name);
     grn_column_name_(ctx, columns[i], &column_name);
     if (((table->header.type == GRN_TABLE_HASH_KEY ||
-          table->header.type == GRN_TABLE_PAT_KEY) &&
+          table->header.type == GRN_TABLE_PAT_KEY ||
+          table->header.type == GRN_TABLE_DAT_KEY) &&
          GRN_TEXT_LEN(&column_name) == 3 &&
          !memcmp(GRN_TEXT_VALUE(&column_name), "_id", 3)) ||
         (table->header.type == GRN_TABLE_NO_KEY &&
@@ -1773,6 +1803,7 @@ dump_table(grn_ctx *ctx, grn_obj *outbuf, grn_obj *table,
   switch (table->header.type) {
   case GRN_TABLE_HASH_KEY:
   case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
     domain = grn_ctx_at(ctx, table->header.domain);
     if (domain) {
       default_flags |= domain->header.flags;
@@ -1851,6 +1882,7 @@ dump_scheme(grn_ctx *ctx, grn_obj *outbuf)
         switch (object->header.type) {
         case GRN_TABLE_HASH_KEY:
         case GRN_TABLE_PAT_KEY:
+        case GRN_TABLE_DAT_KEY:
         case GRN_TABLE_NO_KEY:
         case GRN_TABLE_VIEW:
           dump_table(ctx, outbuf, object, &pending_columns);
@@ -2031,6 +2063,7 @@ proc_check(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
     case GRN_TABLE_PAT_KEY :
       grn_pat_check(ctx, (grn_pat *)obj);
       break;
+    case GRN_TABLE_DAT_KEY :
     case GRN_TABLE_HASH_KEY :
     case GRN_TABLE_NO_KEY :
     case GRN_COLUMN_FIX_SIZE :
@@ -2132,8 +2165,9 @@ proc_truncate(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
           "no such table: <%.*s>", table_name, table_name_len);
     } else {
       switch (table->header.type) {
-      case GRN_TABLE_PAT_KEY :
       case GRN_TABLE_HASH_KEY :
+      case GRN_TABLE_PAT_KEY :
+      case GRN_TABLE_DAT_KEY :
       case GRN_TABLE_NO_KEY :
         grn_table_truncate(ctx, table);
         break;
