@@ -63,12 +63,14 @@ static char bind_address[HOST_NAME_MAX];
 static char hostname[HOST_NAME_MAX];
 static int port = DEFAULT_PORT;
 static int batchmode;
+static int number_of_lines = 0;
 static int newdb;
 static int useql;
 static int (*do_client)(int argc, char **argv);
 static int (*do_server)(char *path);
 static uint32_t default_max_nfthreads = DEFAULT_MAX_NFTHREADS;
-static const char *pidfile_path;
+static const char *pidfile_path = NULL;
+static const char *input_path = NULL;
 
 #ifdef HAVE_LIBEDIT
 #include <locale.h>
@@ -174,11 +176,12 @@ show_version(void)
 
 #define BUFSIZE 0x1000000
 
-inline static int
-prompt(grn_ctx *ctx, char *buf)
+inline static grn_rc
+prompt(grn_ctx *ctx, grn_obj *buf)
 {
   static int the_first_read = GRN_TRUE;
   int len;
+  grn_rc rc;
   if (!batchmode) {
 #ifdef HAVE_LIBEDIT
     const wchar_t *es;
@@ -201,29 +204,38 @@ prompt(grn_ctx *ctx, char *buf)
     }
 #else
     fprintf(stderr, "> ");
-    if (fgets(buf, BUFSIZE, stdin)) {
-      len = strlen(buf);
-    } else {
-      len = 0;
-    }
+    rc = grn_text_fgets(ctx, buf, stdin);
 #endif
   } else {
-    if (fgets(buf, BUFSIZE, stdin)) {
-      len = strlen(buf);
-    } else {
-      len = 0;
+    rc = grn_text_fgets(ctx, buf, stdin);
+    if (rc != GRN_END_OF_DATA) {
+      number_of_lines++;
     }
   }
   if (the_first_read && len > 0) {
     const char bom[] = {0xef, 0xbb, 0xbf};
     if (GRN_CTX_GET_ENCODING(ctx) == GRN_ENC_UTF8 &&
-        len > 3 && !memcmp(buf, bom, 3)) {
-      memmove(buf, buf + 3, len - 3);
-      len -= 3;
+        GRN_TEXT_LEN(buf) > 3 && !memcmp(GRN_TEXT_VALUE(buf), bom, 3)) {
+      grn_obj buf_without_bom;
+      GRN_TEXT_INIT(&buf_without_bom, 0);
+      GRN_TEXT_PUT(ctx, &buf_without_bom,
+                   GRN_TEXT_VALUE(buf) + 3, GRN_TEXT_LEN(buf) - 3);
+      GRN_TEXT_SET(ctx, buf,
+                   GRN_TEXT_VALUE(&buf_without_bom),
+                   GRN_TEXT_LEN(&buf_without_bom));
+      grn_obj_unlink(ctx, &buf_without_bom);
     }
     the_first_read = GRN_FALSE;
   }
-  return len;
+  if (GRN_TEXT_LEN(buf) > 0 &&
+      GRN_TEXT_VALUE(buf)[GRN_TEXT_LEN(buf) - 1] == '\n') {
+    grn_bulk_truncate(ctx, buf, GRN_TEXT_LEN(buf) - 1);
+  }
+  if (GRN_TEXT_LEN(buf) > 0 &&
+      GRN_TEXT_VALUE(buf)[GRN_TEXT_LEN(buf) - 1] == '\r') {
+    grn_bulk_truncate(ctx, buf, GRN_TEXT_LEN(buf) - 1);
+  }
+  return rc;
 }
 
 typedef enum {
@@ -502,6 +514,7 @@ print_return_code(grn_ctx *ctx, grn_rc rc, grn_obj *head, grn_obj *body, grn_obj
       GRN_TEXT_PUTC(ctx, head, ',');
       grn_text_esc(ctx, head, ctx->errbuf, strlen(ctx->errbuf));
       if (ctx->errfunc && ctx->errfile) {
+        grn_obj *command;
         /* TODO: output backtrace */
         GRN_TEXT_PUTS(ctx, head, ",[[");
         grn_text_esc(ctx, head, ctx->errfunc, strlen(ctx->errfunc));
@@ -509,6 +522,19 @@ print_return_code(grn_ctx *ctx, grn_rc rc, grn_obj *head, grn_obj *body, grn_obj
         grn_text_esc(ctx, head, ctx->errfile, strlen(ctx->errfile));
         GRN_TEXT_PUTC(ctx, head, ',');
         grn_text_itoa(ctx, head, ctx->errline);
+        if ((command = GRN_CTX_USER_DATA(ctx)->ptr)) {
+          GRN_TEXT_PUTC(ctx, head, ',');
+          if (input_path) {
+            grn_text_esc(ctx, head, input_path, strlen(input_path));
+          } else {
+            const char *stdin_name = "(stdin)";
+            grn_text_esc(ctx, head, stdin_name, strlen(stdin_name));
+          }
+          GRN_TEXT_PUTC(ctx, head, ',');
+          grn_text_itoa(ctx, head, number_of_lines);
+          GRN_TEXT_PUTC(ctx, head, ',');
+          grn_text_esc(ctx, head, GRN_TEXT_VALUE(command), GRN_TEXT_LEN(command));
+        }
         GRN_TEXT_PUTS(ctx, head, "]]");
       }
     }
@@ -613,6 +639,7 @@ s_output(grn_ctx *ctx, int flags, void *arg)
 {
   if (ctx && ctx->impl && (flags & GRN_CTX_TAIL)) {
     grn_obj *buf = ctx->impl->outbuf;
+    grn_obj *command;
     if (GRN_TEXT_LEN(buf) || ctx->rc) {
       FILE * stream = (FILE *) arg;
       grn_obj head, foot;
@@ -628,6 +655,8 @@ s_output(grn_ctx *ctx, int flags, void *arg)
       GRN_OBJ_FIN(ctx, &head);
       GRN_OBJ_FIN(ctx, &foot);
     }
+    command = GRN_CTX_USER_DATA(ctx)->ptr;
+    GRN_BULK_REWIND(command);
   }
 }
 
@@ -642,27 +671,25 @@ do_alone(int argc, char **argv)
   if (argc > 0 && argv) { path = *argv++; argc--; }
   db = (newdb || !path) ? grn_db_create(ctx, path, NULL) : grn_db_open(ctx, path);
   if (db) {
+    grn_obj command;
+    GRN_TEXT_INIT(&command, 0);
+    GRN_CTX_USER_DATA(ctx)->ptr = &command;
     grn_ctx_recv_handler_set(ctx, s_output, stdout);
     if (!argc) {
       grn_obj text;
       GRN_TEXT_INIT(&text, 0);
-      rc = grn_bulk_reserve(ctx, &text, BUFSIZE);
-      if (!rc) {
-        char *buf = GRN_TEXT_VALUE(&text);
-        int  len;
-        while ((len = prompt(ctx, buf))) {
-          uint32_t size = len - 1;
-          grn_ctx_send(ctx, buf, size, 0);
-          if (ctx->stat == GRN_CTX_QUIT) { break; }
-        }
-        rc = ctx->rc;
-      } else {
-        fprintf(stderr, "grn_bulk_reserve() failed (%d): %d\n", BUFSIZE, rc);
+      while (prompt(ctx, &text) != GRN_END_OF_DATA) {
+        GRN_TEXT_PUT(ctx, &command, GRN_TEXT_VALUE(&text), GRN_TEXT_LEN(&text));
+        grn_ctx_send(ctx, GRN_TEXT_VALUE(&text), GRN_TEXT_LEN(&text), 0);
+        GRN_BULK_REWIND(&text);
+        if (ctx->stat == GRN_CTX_QUIT) { break; }
       }
+      rc = ctx->rc;
       grn_obj_unlink(ctx, &text);
     } else {
       rc = grn_ctx_sendv(ctx, argc, argv, 0);
     }
+    grn_obj_unlink(ctx, &command);
     grn_obj_close(ctx, db);
   } else {
     fprintf(stderr, "db open failed (%s): %s\n", path, ctx->errbuf);
@@ -717,20 +744,13 @@ g_client(int argc, char **argv)
     if (!argc) {
       grn_obj text;
       GRN_TEXT_INIT(&text, 0);
-      rc = grn_bulk_reserve(ctx, &text, BUFSIZE);
-      if (!rc) {
-        char *buf = GRN_TEXT_VALUE(&text);
-        int   len;
-        while ((len = prompt(ctx, buf))) {
-          uint32_t size = len - 1;
-          grn_ctx_send(ctx, buf, size, 0);
-          rc = ctx->rc;
-          if (rc) { break; }
-          if (c_output(ctx)) { goto exit; }
-          if (ctx->stat == GRN_CTX_QUIT) { break; }
-        }
-      } else {
-        fprintf(stderr, "grn_bulk_reserve() failed (%d): %d\n", BUFSIZE, rc);
+      while (prompt(ctx, &text) != GRN_END_OF_DATA) {
+        grn_ctx_send(ctx, GRN_TEXT_VALUE(&text), GRN_TEXT_LEN(&text), 0);
+        GRN_BULK_REWIND(&text);
+        rc = ctx->rc;
+        if (rc) { break; }
+        if (c_output(ctx)) { goto exit; }
+        if (ctx->stat == GRN_CTX_QUIT) { break; }
       }
       grn_obj_unlink(ctx, &text);
     } else {
@@ -2035,7 +2055,10 @@ show_config(FILE *out, const grn_str_getopt_opt *opts, int flags)
     switch (o->op) {
     case getopt_op_none:
       if (o->arg && *o->arg) {
-        if (o->longopt && strcmp(o->longopt, "config-path")) {
+        if (o->longopt &&
+            strcmp(o->longopt, "config-path") != 0 &&
+            strcmp(o->longopt, "address") != 0 &&
+            strcmp(o->longopt, "admin-html-path") != 0) {
           fprintf(out, "%s=%s\n", o->longopt, *o->arg);
         }
       }
@@ -2086,7 +2109,6 @@ main(int argc, char **argv)
     *cache_limitstr = NULL, *admin_html_path = NULL, *command_versionstr = NULL,
     *match_escalation_thresholdstr = NULL;
   const char *config_path = NULL;
-  const char *input_path = NULL;
   int r, i, mode = mode_alone;
   static grn_str_getopt_opt opts[] = {
     {'p', "port", NULL, 0, getopt_op_none},
