@@ -1,5 +1,5 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2010- Brazil
+/* Copyright(C) 2010-2011 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -37,6 +37,9 @@
 #include <groonga.h>
 #include <pthread.h>
 
+/* groonga origin headers */
+#include <str.h>
+
 #include "util.h"
 
 #define DEFAULT_PORT 8080
@@ -51,6 +54,17 @@ grn_rc grn_ctx_close(grn_ctx *ctx);
 #define LOG_SPLIT_LINES 1000000
 #define MAX_THREADS 128 /* max 256 */
 
+typedef enum {
+  run_mode_none = 0,
+  run_mode_usage,
+  run_mode_daemon,
+  run_mode_error
+} run_mode;
+
+#define RUN_MODE_MASK                0x007f
+#define RUN_MODE_ENABLE_MAX_FD_CHECK 0x0080
+
+
 typedef struct {
   grn_ctx *ctx;
   grn_obj *db;
@@ -61,7 +75,7 @@ typedef struct {
   struct event_base *base;
   struct evhttp *httpd;
   struct event pulse;
-  const char *log_path;
+  const char *log_base_path;
   FILE *log_file;
   uint32_t log_count;
 } thd_data;
@@ -113,8 +127,9 @@ suggest_result(struct evbuffer *res_buf, const char *types, const char *query,
   }
 }
 
-static int
-log_send(struct evbuffer *res_buf, thd_data *thd, struct evkeyvalq *get_args)
+static void
+log_send(struct evkeyvalq *output_headers, struct evbuffer *res_buf,
+         thd_data *thd, struct evkeyvalq *get_args)
 {
   uint64_t millisec;
   int threshold, limit;
@@ -191,17 +206,27 @@ log_send(struct evbuffer *res_buf, thd_data *thd, struct evkeyvalq *get_args)
   {
     int content_length;
     if (callback) {
+      evhttp_add_header(output_headers,
+                        "Content-Type", "text/javascript; charset=UTF-8");
       content_length = strlen(callback);
       evbuffer_add(res_buf, callback, content_length);
       evbuffer_add(res_buf, "(", 1);
       content_length += suggest_result(res_buf, types, query, target_name,
-        threshold, limit, &(thd->cmd_buf), thd->ctx) + 3;
+                                       threshold, limit,
+                                       &(thd->cmd_buf), thd->ctx) + 3;
       evbuffer_add(res_buf, ");", 2);
     } else {
+      evhttp_add_header(output_headers,
+                        "Content-Type", "application/json; charset=UTF-8");
       content_length = suggest_result(res_buf, types, query, target_name,
-        threshold, limit, &(thd->cmd_buf), thd->ctx);
+                                      threshold, limit,
+                                      &(thd->cmd_buf), thd->ctx);
+    }
+    if (content_length >= 0) {
+      char num_buf[16];
+      snprintf(num_buf, 16, "%d", content_length);
+      evhttp_add_header(output_headers, "Content-Length", num_buf);
     }
-    return content_length;
   }
 }
 
@@ -247,23 +272,14 @@ generic_handler(struct evhttp_request *req, void *arg)
       err(1, "failed to create response buffer");
     }
 
-    evhttp_add_header(req->output_headers,
-      "Content-Type", "text/javascript; charset=UTF-8");
     evhttp_add_header(req->output_headers, "Connection", "close");
 
-    {
-      int content_length = log_send(res_buf, thd, &args);
-      if (content_length >= 0) {
-        char num_buf[16];
-        snprintf(num_buf, 16, "%d", content_length);
-        evhttp_add_header(req->output_headers, "Content-Length", num_buf);
-      }
-    }
+    log_send(req->output_headers, res_buf, thd, &args);
     evhttp_send_reply(req, HTTP_OK, "OK", res_buf);
     evbuffer_free(res_buf);
     /* logging */
     {
-      if (thd->log_path) {
+      if (thd->log_base_path) {
         if (!thd->log_file) {
           time_t n;
           struct tm *t_st;
@@ -273,7 +289,7 @@ generic_handler(struct evhttp_request *req, void *arg)
           t_st = localtime(&n);
 
           snprintf(p, PATH_MAX, "%s%04d%02d%02d%02d%02d%02d-%02d",
-            thd->log_path,
+            thd->log_base_path,
             t_st->tm_year + 1900, t_st->tm_mon + 1, t_st->tm_mday,
             t_st->tm_hour, t_st->tm_min, t_st->tm_sec, thd->thread_id);
 
@@ -504,7 +520,7 @@ recv_from_learner(void *arg)
 static int
 serve_threads(int nthreads, int port, const char *db_path, void *zmq_ctx,
               const char *send_endpoint, const char *recv_endpoint,
-              const char *log_path)
+              const char *log_base_path)
 {
   int nfd;
   uint32_t i;
@@ -547,7 +563,7 @@ serve_threads(int nthreads, int port, const char *db_path, void *zmq_ctx,
             print_error("error in grn_db_open() on thread %d.", i);
           } else {
             GRN_TEXT_INIT(&(thds[i].cmd_buf), 0);
-            thds[i].log_path = log_path;
+            thds[i].log_base_path = log_base_path;
             thds[i].thread_id = i;
             evhttp_set_gencb(thds[i].httpd, generic_handler, &thds[i]);
             evhttp_set_timeout(thds[i].httpd, 10);
@@ -616,96 +632,131 @@ get_core_number(void)
 static void
 usage(FILE *output)
 {
-  fprintf(output,
-          "Usage: groonga-suggest-httpd [options...] db_path\n"
-          "db_path:\n"
-          "  specify groonga database path which is used for suggestion.\n"
-          "options:\n"
-          "  -p <port number>   : http server port number (default: %d)\n"
-          "  -c <thread number> : server thread number (default: %d)\n"
-          "  -s <send endpoint> : send endpoint (ex. tcp://example.com:1234)\n"
-          "  -r <recv endpoint> : recv endpoint (ex. tcp://example.com:1235)\n"
-          "  -l <path prefix>   : log path prefix\n"
-          "  -d                 : daemonize\n",
-          DEFAULT_PORT, default_max_threads);
+  fprintf(
+    output,
+    "Usage: groonga-suggest-httpd [options...] db_path\n"
+    "db_path:\n"
+    "  specify groonga database path which is used for suggestion.\n"
+    "\n"
+    "options:\n"
+    "  -p, --port <port number>                  : http server port number\n"
+    "                                              (default: %d)\n"
+    /*
+    "  --address <ip/hostname>                   : server address to listen\n"
+    "                                              (default: %s)\n"
+    */
+    "  -c <thread number>                        : number of server threads\n"
+    "                                              (deprecated. use --n-threads)\n"
+    "  -t, --n-threads <thread number>           : number of server threads\n"
+    "                                              (default: %d)\n"
+    "  -s, --send-endpoint <send endpoint>       : send endpoint\n"
+    "                                              (ex. tcp://example.com:1234)\n"
+    "  -r, --receive-endpoint <receive endpoint> : receive endpoint\n"
+    "                                              (ex. tcp://example.com:1235)\n"
+    "  -l, --log-base-path <path prefix>         : log path prefix\n"
+    "  -d, --daemon                              : daemonize\n"
+    "  --disable-max-fd-check                    : disable max FD check on start\n"
+    "  -h, --help                                : show this message\n",
+    DEFAULT_PORT, default_max_threads);
 }
 
 int
 main(int argc, char **argv)
 {
   int port_no = DEFAULT_PORT, daemon = 0;
-  const char *send_endpoint = NULL, *recv_endpoint = NULL, *log_path = NULL;
+  const char *max_threads_string = NULL, *port_string = NULL;
+  const char *address;
+  const char *send_endpoint = NULL, *recv_endpoint = NULL, *log_base_path = NULL;
+  int n_processed_args, flags = RUN_MODE_ENABLE_MAX_FD_CHECK;
+  run_mode mode = run_mode_none;
 
-  /* check environment */
-  {
-    struct rlimit rlim;
-    if (!getrlimit(RLIMIT_NOFILE, &rlim)) {
-      if (rlim.rlim_max < MIN_MAX_FDS) {
-        print_error("too small max fds. %d required.", MIN_MAX_FDS);
-        return -1;
-      }
-      rlim.rlim_cur = rlim.rlim_cur;
-      setrlimit(RLIMIT_NOFILE, &rlim);
-    }
-  }
   if (!(default_max_threads = get_core_number())) {
     default_max_threads = DEFAULT_MAX_THREADS;
   }
 
   /* parse options */
   {
-    int ch;
+    static grn_str_getopt_opt opts[] = {
+      {'c', NULL, NULL, 0, getopt_op_none}, /* deprecated */
+      {'t', "n-threads", NULL, 0, getopt_op_none},
+      {'h', "help", NULL, run_mode_usage, getopt_op_update},
+      {'p', "port", NULL, 0, getopt_op_none},
+      {'\0', "bind-address", NULL, 0, getopt_op_none}, /* not supported yet */
+      {'s', "send-endpoint", NULL, 0, getopt_op_none},
+      {'r', "receive-endpoint", NULL, 0, getopt_op_none},
+      {'l', "log-base-path", NULL, 0, getopt_op_none},
+      {'d', "daemon", NULL, run_mode_daemon, getopt_op_update},
+      {'\0', "disable-max-fd-check", NULL, RUN_MODE_ENABLE_MAX_FD_CHECK,
+       getopt_op_off},
+      {'\0', NULL, NULL, 0, 0}
+    };
+    opts[0].arg = &max_threads_string;
+    opts[1].arg = &max_threads_string;
+    opts[3].arg = &port_string;
+    opts[4].arg = &address;
+    opts[5].arg = &send_endpoint;
+    opts[6].arg = &recv_endpoint;
+    opts[7].arg = &log_base_path;
 
-    while ((ch = getopt(argc, argv, "c:p:s:r:l:d")) != -1) {
-      switch(ch) {
-      case 'c':
-        default_max_threads = atoi(optarg);
-        if (default_max_threads > MAX_THREADS) {
-          print_error("too many threads. limit to %d.", MAX_THREADS);
-          default_max_threads = MAX_THREADS;
-        }
-        break;
-      case 'p':
-        port_no = atoi(optarg);
-        break;
-      case 's':
-        send_endpoint = optarg;
-        break;
-      case 'r':
-        recv_endpoint = optarg;
-        break;
-      case 'l':
-        log_path = optarg;
-        break;
-      case 'd':
-        daemon = 1;
-        break;
-      }
-    }
-    argc -= optind; argv += optind;
+    n_processed_args = grn_str_getopt(argc, argv, opts, &flags);
   }
 
   /* main */
-  if (argc != 1) {
+  mode = (flags & RUN_MODE_MASK);
+  if (n_processed_args < 0 ||
+      (argc - n_processed_args) != 1 ||
+      mode == run_mode_error) {
     usage(stderr);
+    return EXIT_FAILURE;
+  } else if (mode == run_mode_usage) {
+    usage(stdout);
+    return EXIT_SUCCESS;
   } else {
     grn_ctx ctx;
     void *zmq_ctx;
+    int max_threads;
 
-    if (daemon) {
+    if (max_threads_string) {
+      max_threads = atoi(max_threads_string);
+      if (max_threads > MAX_THREADS) {
+        print_error("too many threads. limit to %d.", MAX_THREADS);
+        max_threads = MAX_THREADS;
+      }
+    } else {
+      max_threads = default_max_threads;
+    }
+
+    if (port_string) {
+      port_no = atoi(port_string);
+    }
+
+    if (flags & RUN_MODE_ENABLE_MAX_FD_CHECK) {
+      /* check environment */
+      struct rlimit rlim;
+      if (!getrlimit(RLIMIT_NOFILE, &rlim)) {
+        if (rlim.rlim_max < MIN_MAX_FDS) {
+          print_error("too small max fds. %d required.", MIN_MAX_FDS);
+          return -1;
+        }
+        rlim.rlim_cur = rlim.rlim_cur;
+        setrlimit(RLIMIT_NOFILE, &rlim);
+      }
+    }
+
+    if (mode == run_mode_daemon) {
       daemonize();
     }
 
     grn_init();
     grn_ctx_init(&ctx, 0);
-    if ((db = grn_db_open(&ctx, argv[0]))) {
+    if ((db = grn_db_open(&ctx, argv[n_processed_args]))) {
       if ((zmq_ctx = zmq_init(1))) {
         signal(SIGTERM, signal_handler);
         signal(SIGINT, signal_handler);
         signal(SIGQUIT, signal_handler);
 
-        serve_threads(default_max_threads, port_no, argv[0], zmq_ctx,
-          send_endpoint, recv_endpoint, log_path);
+        serve_threads(max_threads, port_no, argv[n_processed_args], zmq_ctx,
+                      send_endpoint, recv_endpoint, log_base_path);
         zmq_term(zmq_ctx);
       } else {
         print_error("cannot create zmq context.");
