@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2; coding: utf-8 -*- */
 /*
-  Copyright (C) 2010  Kouhei Sutou <kou@clear-code.com>
+  Copyright (C) 2010-2011  Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -178,10 +178,9 @@ cut_teardown(void)
 
   grn_obj_unlink(context, inspected);
 
-  if (context) {
-    grn_ctx_fin(context);
-    g_free(context);
-  }
+  grn_obj_close(context, database);
+  grn_ctx_fin(context);
+  g_free(context);
 
   remove_tmp_directory();
 }
@@ -613,6 +612,8 @@ test_pvector_with_records(void)
   GRN_PTR_PUT(context, pvector, groonga);
   GRN_PTR_PUT(context, pvector, razil);
   inspected = grn_inspect(context, NULL, pvector);
+  grn_obj_unlink(context, groonga);
+  grn_obj_unlink(context, razil);
   cut_assert_equal_string("[\"groonga\",\"razil\"]", inspected_string());
 }
 
@@ -661,8 +662,10 @@ test_accessor_column_name(gconstpointer data)
   object = get_object(table_name);
   accessor = grn_obj_column(context, object,
                             accessor_name, strlen(accessor_name));
+  grn_obj_unlink(context, object);
   cut_assert_not_null(accessor);
   inspected = grn_inspect(context, NULL, accessor);
+  grn_obj_unlink(context, accessor);
   cut_assert_equal_string(accessor_name, inspected_string());
 }
 
@@ -709,6 +712,7 @@ test_accessor_dynamic_pseudo_column_name(gconstpointer data)
                             accessor_name, strlen(accessor_name));
   cut_assert_not_null(accessor);
   inspected = grn_inspect(context, NULL, accessor);
+  grn_obj_unlink(context, accessor);
   cut_assert_equal_string(accessor_name, inspected_string());
 }
 
